<?php
/**
 * @copyright	Copyright (C) 2016 itoctopus. All rights reserved.
 * @date		Wednesday, December 7th, 2016
 * @author		itoctopus
 * @email		development@itoctopus.com
 * @website		http://www.itoctopus.com
 * @license		GNU General Public License version 2 or later
 * @description	This script converts keywords to tags, and tags the appropriate articles. The script is provided for free and as-is.
 */

/**
 * Define the application's minimum supported PHP version as a constant so it can be referenced within the application.
 */
define('JOOMLA_MINIMUM_PHP', '5.3.10');

if (version_compare(PHP_VERSION, JOOMLA_MINIMUM_PHP, '<'))
{
	die('Your host needs to use PHP ' . JOOMLA_MINIMUM_PHP . ' or higher to run this version of Joomla!');
}

// Saves the start time and memory usage.
$startTime = microtime(1);
$startMem  = memory_get_usage();

/**
 * Constant that is checked in included files to prevent direct access.
 * define() is used in the installation folder rather than "const" to not error for PHP 5.2 and lower
 */
define('_JEXEC', 1);

if (file_exists(__DIR__ . '/defines.php'))
{
	
	include_once __DIR__ . '/defines.php';
}

if (!defined('_JDEFINES'))
{
	define('JPATH_BASE', __DIR__);
	require_once JPATH_BASE . '/includes/defines.php';
}

require_once JPATH_BASE . '/includes/framework.php';

// Instantiate the application.
$app = JFactory::getApplication('site');

// No error reporting and no maximum execution time
error_reporting(0);
ini_set('max_execution_time', 0);
set_time_limit(0);
ob_start();


// Get a handle on the database object
$db = JFactory::getDbo();


/* The following function is copied from the JHelperTags (which is located in the libraries/cms/helpers/tags.php file). 
   The reason we were not able to use the function "as-is" is because it requires for a user to be logged in to create the tag, so we needed to remove that condition since we wanted to run it externally. */
function createTagsFromFieldExternal($tags)
{
	if (empty($tags) || $tags[0] == '')
	{
		return;
	}
	else
	{
		// We will use the tags table to store them
		JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_tags/tables');
		$tagTable  = JTable::getInstance('Tag', 'TagsTable');
		$newTags   = array();
		$canCreate = true;
		foreach ($tags as $key => $tag)
		{
			// User is not allowed to create tags, so don't create.
			if (strpos($tag, '#new#') !== false && !$canCreate)
			{
				continue;
			}

			// Remove the #new# prefix that identifies new tags
			$tagText = str_replace('#new#', '', $tag);

			if ($tagText == $tag)
			{
				$newTags[] = (int) $tag;
			}
			else
			{
				// Clear old data if exist
				$tagTable->reset();

				// Try to load the selected tag
				if ($tagTable->load(array('title' => $tagText)))
				{
					$newTags[] = (int) $tagTable->id;
				}
				else
				{
					// Prepare tag data
					$tagTable->id = 0;
					$tagTable->title = $tagText;
					$tagTable->published = 1;

					// $tagTable->language = property_exists ($item, 'language') ? $item->language : '*';
					$tagTable->language = '*';
					$tagTable->access = 1;

					// Make this item a child of the root tag
					$tagTable->setLocation($tagTable->getRootId(), 'last-child');
					
					

					// Try to store tag
					if ($tagTable->check())
					{
						// Assign the alias as path (autogenerated tags have always level 1)
						$tagTable->path = $tagTable->alias;

						if ($tagTable->store())
						{
							
							$newTags[] = (int) $tagTable->id;
						}
					}
				}
			}
		}
		$result = $newTags;
	}
	
	return $result;
}


// Get all the articles that have meta keywords from the database - you may want to use LIMIT if you have a huge number of articles.
$sql = "SELECT `id`, `title`, `metakey` FROM `#__content` WHERE state=1 AND `metakey` != ''"; // add "LIMIT 0, 100" just after '' if the script is timing out on you, and then, change that to "LIMIT 100, 100" and then "LIMIT 200, 100", etc...
$db->setQuery($sql);
$arrArticles = $db->loadAssocList();

// Now that we have all the meta keywords, we will need to loop through all of them and then add them as tags (if they don't already exist as tags) and then add the association between the tag and the article
for ($i = 0; $i < count($arrArticles); $i++){
	echo('<em>Tagging</em> '.$arrArticles[$i]['title']. ' <em>(ID # </em><strong>'.$arrArticles[$i]['id'].'</strong>)...');
	flush();
	ob_flush();
	// Explode the meta keywords into an array
	$arrMetaKey = explode(', ', $arrArticles[$i]['metakey']);
	// Reset the new tags array
	$arrNewTags = array();
	// Add '#new#' to each item so that Joomla processes it
	for ($j = 0; $j < count($arrMetaKey); $j++){
		$arrNewTags[] = '#new#'.$arrMetaKey[$j];
	}
	
	// Effectively create the tags in the database (they will only be created if they are new
	$objHelperTags = new JHelperTags;
	$objHelperTags->typeAlias = 'com_content.article';
	$arrTagIDs = createTagsFromFieldExternal($arrNewTags);
	
	// Tag the article with the tags returned from the previous step
	$objContentJTable = JTable::getInstance('content');
	$objContentJTable->load($arrArticles[$i]['id']);
	$objHelperTags->tagItem(0, $objContentJTable, $arrTagIDs);
	echo('<em>done!</em><br />');
}

echo('<br /><br /><strong>******* Finished Tagging Articles! *******</strong>');